package internal

import (
	"context"
	"time"

	corev1 "k8s.io/api/core/v1"
	"k8s.io/apimachinery/pkg/fields"
	icorev1 "k8s.io/client-go/informers/core/v1"
	"k8s.io/client-go/kubernetes"
	"k8s.io/client-go/tools/cache"
)

func NewInformer(ctx context.Context, config Config, client *kubernetes.Clientset, manager *Manager) cache.SharedIndexInformer {
	informer := icorev1.NewPodInformer(client, config.Namespace, time.Second*30, cache.Indexers{cache.NamespaceIndex: cache.MetaNamespaceIndexFunc})
	informer.AddEventHandler(cache.FilteringResourceEventHandler{
		FilterFunc: func(obj interface{}) bool {
			pod := obj.(*corev1.Pod)
			selector := fields.AndSelectors(
				fields.OneTermEqualSelector("app.kubernetes.io/name", "redis"),
				fields.OneTermEqualSelector("app.kubernetes.io/instance", config.Instance))
			f := fields.Set(pod.Labels)
			return selector.Matches(f) && pod.Name != config.Hostname && pod.Status.PodIP != ""
		},
		Handler: cache.ResourceEventHandlerFuncs{
			AddFunc: func(obj interface{}) {
				pod := obj.(*corev1.Pod)
				if !pod.ObjectMeta.DeletionTimestamp.IsZero() {
					return
				}
				manager.AddReplica(ctx, pod)
			},
			UpdateFunc: func(oldObj, newObj interface{}) {
				pod := newObj.(*corev1.Pod)
				if !pod.ObjectMeta.DeletionTimestamp.IsZero() {
					manager.RemoveReplica(ctx, pod)
					return
				}
				replica := manager.GetReplica(pod.Name)
				if replica != pod.Status.PodIP {
					if replica != "" {
						manager.RemoveReplica(ctx, pod)
					}
					manager.AddReplica(ctx, pod)
				}
			},
			DeleteFunc: func(obj interface{}) {
				pod := obj.(*corev1.Pod)
				manager.RemoveReplica(ctx, pod)
			},
		}})
	return informer
}
