package internal

import (
	"context"
	"errors"
	"fmt"
	"net/http"

	"github.com/go-redis/redis/v8"
	corev1 "k8s.io/api/core/v1"
)

type Manager struct {
	replicas map[string]string
	rdb      *redis.Client
}

func NewManager(config Config) *Manager {
	return &Manager{
		replicas: make(map[string]string),
		rdb: redis.NewClient(&redis.Options{
			Password: config.Pass,
		}),
	}
}

func (m *Manager) AddReplica(ctx context.Context, pod *corev1.Pod) {
	_, err := m.rdb.Do(ctx, "replicaof", pod.Status.PodIP, "6379").Result()
	if err != nil {
		fmt.Printf("failed to add %s (%s)\n", pod.Status.PodIP, pod.Name)
		fmt.Println(err)
		return
	}
	m.replicas[pod.Name] = pod.Status.PodIP
	fmt.Printf("added %s (%s)\n", pod.Status.PodIP, pod.Name)
}

func (m *Manager) RemoveReplica(ctx context.Context, pod *corev1.Pod) {
	if m.replicas[pod.Name] == "" {
		return
	}
	_, err := m.rdb.Do(ctx, "replicaof", "remove", pod.Status.PodIP, "6379").Result()
	if err != nil {
		fmt.Printf("failed to remove %s (%s)\n", pod.Status.PodIP, pod.Name)
		fmt.Println(err)
		return
	}
	delete(m.replicas, pod.Name)
	fmt.Printf("removed %s (%s)\n", pod.Status.PodIP, pod.Name)
}

func (m *Manager) GetReplica(name string) string {
	return m.replicas[name]
}

func (m *Manager) Check(req *http.Request) error {
	res := m.rdb.Ping(req.Context())
	if res.Err() != nil {
		return res.Err()
	}
	if res.Val() != "PONG" {
		return errors.New("expected PONG reply")
	}
	return nil
}
